; Patch BOSLoader for further improvements.
;
; Jason Hood, 1 to 10 March, 2016 (originally 19 & 20 December, 2013).
;
; This patch provides further enhancements to the Balls of Steel Loader:
;
; * work with other Wildfire pinballs;
; * display the help screen;
; * delay on F1 to stop the help screen immediately exiting;
; * set the window's client area to the table resolution;
; * start full screen by default, unless file "windowed" is in the current dir;
; * prevent F from toggling full screen.

<$bosloader.0>

; Link to my website, which links to the original.
<$$260E3> "bos.adoxa.vze.com/\0"

; Add myself to the credits.
<$$26128..$$261D4>
	"BOS Loader v1.1\n"
	"-----\n"
	"\n"
	"Balls of Steel is  1997 3D Realms.\n"
	"BOS Loader is  2010 Christophe Thibault.\n"
	"This version by Jason Hood.\n"
	"\n"
	"Use Alt+Enter to toggle full screen mode.\0"


; Devil's Island 1.2RN and Austin Powers use a mutex to prevent running the
; tables directly - create that mutex.
<$$1E49> jmp	@Mutex


; The help screen immediately exits because F1 is immediately repeated.  Add a
; small delay to avoid that.  It was fixed in Devil's Island 1.2RN and Austin
; Powers, but won't hurt to do it anyway.
<$$161C> jmp	@F1

; Place in the unused part of the .text section.
<$$53E5..$$5400>
@F1:
	jz	$$1634
	cmp	esi,70			;; F1 (replacing F)
	jne	short @f
	call	@data
	mov	[eax+20004],esi 	;; flag delay on palette change
@@:	jmp	$$1642


; Size the window to the make the client the resolution.
<$$152C> call @WindowSize

<$$1546>
	push	edi
	push	ebp
	jmp	short $$1550


; Only hook DirectDrawCreate for the tables and Balls of Steel menu (the other
; menus are normal windows).  The tables are compiled with Watcom, the menus
; are not; Watcom can be detected by an 80 byte after the DOS signature.
<$$1B78..$$1BDA>
	cmp	byte[eax+2],80
	mov	esi,[$$4E68]
	je	short @table
	push	eax
	call	dword[$$240AC]		;; GetModuleFileName
	mov	ebx,[$$24240]		;; strstr
	lea	edx,[ebp-218]
	push	$$26310 		;; "bosloader.exe"
	push	edx
	call	ebx
	add	esp,8
	test	eax,eax
	jnz	$$1C80
	;lea	 eax,[ebp-218]
	push	$$26308 		;; "bos.exe"
	;push	 eax
	sub	esp,4
	call	ebx
	add	esp,8
	test	eax,eax
	jz	short @hook
	nop				;; back to original relocations
@table:
	sete	[$$27258]
	push	edi
	push	$$2714C
	push	$$19D0
	push	$$26278 		;; "DirectDrawCreate"
	push	$$2626C 		;; "DDRAW.DLL"
	call	esi
@hook:

<$$1C4E>
	push	$$271E0
	push	$$1190
	push	$$262F8 		;; "SetWindowPos"
	push	$$262EC 		;; "USER32.DLL"

<$$26308> "bos.exe"
<$$2C11A> dw	3B7E,3B87,3B8D,3B98,3BAB


; SetWindowPos hook - only handle the forms of the Balls of Steel menu.
<$$11A3..$$11E4>
	lea	eax,[esp+78]
	push	0FFF
	push	eax
	push	0
	call	dword[$$240AC]		;; GetModuleFileName
	lea	ecx,[esp+18]
	push	eax
	push	6F
	push	ecx
	push	esi
	call	dword[$$242D8]		;; GetClassName
	mov	edi,[$$24250]		;; stricmp
	pop	eax
	lea	edx,[esp+78+eax-7]
	push	$$26308 		;; "bos.exe"
	push	edx
	call	edi
	add	esp,8
	test	eax,eax
	jnz	$$1346
	!pad	90

<$$2C030> dw	31B1,31C0,31C6,31D0


; Don't make full screen topmost.
<$$22A2> db	0FE

; Start the table full screen.
<$$2602> jmp	@Windowed
<$$2640> jmp	@Windowed


; Place in the unused part of the CODE section.
<$$230AC..$$23200>
@Mutex:
	mov	eax,[esp]
	cmp	dword[eax-1C80+27258],0
	je	short @ret
	call	@f
	"BOS menu\0"
@@:	push	0
	push	0
	call	$$9A64
@ret:	ret


@800:
	mov	ecx,800.		;; wrap the 640x480 BOS menu in 800x600
	mov	dword[ebx],600.
	jmp	short @f

; Get the window size so the client rectangle is the resolution.  The window is
; initially created without a sizing frame (the BOS menu cannot be resized);
; the sizing style is added when the table is started, but not refreshed until
; a resize occurs.  This means the window has a different size depending if it
; starts windowed or not.  I've left it this way so if your desktop matches the
; resolution, you still get a full window (a sizing frame will make it smaller
; so the window fits the desktop), but it does mean starting windowed, switching
; full screen and back to window will leave you with a slightly smaller client.
@WindowSize:
	mov	[eax],ebx		;; EAX is the LPRECT used for centering
	mov	[eax+4],ebx		;; EBX is 0
	mov	ebx,[esp]		;; return address to relocate
	xor	bx,bx
	add	ebx,27240
	mov	ecx,[ebx-10]		;; width
	jecxz	@800			;; for the BOS menu
@@:	mov	edx,[ebx]		;; height
	mov	[eax+8],ecx
	mov	[eax+0C],edx
	push	eax
	push	0			;; no menu
	push	0C40000 		;; WS_CAPTION | WS_THICKFRAME
	call	@WantFull
	je	short @f
	mov	byte[esp+2],0C0 	;; WS_CAPTION
@@:	push	eax
	lea	eax,[ebx-7240+62EC]	;; "USER32.DLL"
	push	eax
	call	esi			;; GetModuleHandle
	call	@f
	"AdjustWindowRect\0"
@@:	push	eax
	call	$$9BBC			;; GetProcAddress
	call	eax
	pop	eax
	mov	ebp,[eax+8]
	mov	edi,[eax+0C]
	sub	ebp,[eax]
	sub	edi,[eax+4]
	mov	edx,[ebx]		;; center the client, not the window
	sub	edx,[eax+4]
	sub	edx,[eax+4]
	mov	[esp+0C],ebp		;; update the centering sizes
	mov	[esp+10],edx

	xor	ebx,ebx
	jmp	$$1390

; Don't start full screen if "windowed" exists in the current directory.
@WantFull:
	push	eax
	call	@f
	"windowed\0"
@@:	call	$$9B6C			;; GetFileAttributes
	inc	eax
	pop	eax
	ret

@Windowed:
	call	@WantFull
	jne	short @f
	call	$$2230
@@:	jmp	$$2690


; The palette is set to all black, the surface is locked, drawn and unlocked,
; then the final palette is set, which bosloader fails to update.  Detect the
; black palette and do the update on the next palette change.
@PaletteSetEntries:
	push	esi
	push	edi
	mov	edi,[esp+18+4]
	xor	eax,eax
	mov	ecx,100
	push	edi
	repe	scasd
	pop	edi
	je	short @black
	mov	esi,[esp+14]
	lea	eax,[edi+2]
	mov	edi,[esp+1C-4]
	mov	ecx,[esp+10-4]
	lea	esi,[esi*4+ecx+4]
@@:
	xor	edx,edx
	xor	ecx,ecx
	mov	dh,[eax-2]
	mov	cl,[eax]
	mov	dl,[eax-1]
	add	eax,4
	shl	edx,8
	or	edx,ecx
	mov	[esi],edx
	add	esi,4
	dec	edi
	jnz	short @b
	call	$$26C0
	call	@data			;; delay if F1 was pressed
	cmp	[eax+4],edi
	jz	short @done
	mov	[eax+4],edi
	push	250.
	call	$$9CD4			;; Sleep
@done:	pop	edi
	pop	esi
	xor	eax,eax
	ret	14

@black:
	; Let Unlock know it should skip the update.
	call	@data
	mov	[eax],ah
	jmp	short @done

@data:
	mov	eax,[esp]
	mov	ax,9300 		;; unused part of the DATA section
	ret

; myIDDrawSurface1::Unlock
<$$47E0>
	mov	ecx,[esp+04]
	cmp	ecx,[$$27150]
	je	short @f
	xor	eax,eax
	ret	8
	db	90,90			;; padding to keep the relocation
@@:	mov	eax,$$29300
	cmp	[eax],al
	mov	[eax],al
	nop

<$$24404> dd	@PaletteSetEntries
